<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Ratctreceber;
use Teknisa\FIN\Modules\ContaReceber\Factory\RateioContaFactory as RateioContaFactory;

/**
 * Class RateioContaService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class RateioContaService {

    protected $dtsAjustesBoleto;
    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     * @param RateioContaFactory $rateioContaFactory
     */
    function __construct(EntityManager $entityManager, RateioContaFactory $rateioContaFactory)
    {
        $this->entityManager = $entityManager;
        $this->rateioContaFactory = $rateioContaFactory;
    }

    /**
     * Finaliza a Inserção e/ou Atualização do(s) Rateio(s) Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $rateios
     * @throws \Exception
     */
    public function finalizaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $rateios
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Rateios Conta Receber
            $ratctreceber = $this->rateioContaFactory->buscaRateiosConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec
            );

            // Percorre Rateio(s) Antigo(s) da Conta a Receber
            foreach($ratctreceber as $rateio)
                // Deleta Rateio Conta Receber
                $this->entityManager->remove($rateio);
            $this->entityManager->flush();

            // Percorre Rateios da Conta a Receber
            foreach($rateios as $rateio) {

                // Insere Rateio Conta Receber
                $ratctreceber = $this->rateioContaFactory->insereRateioConta(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $rateio['CDFIRATCTREC'], $rateio['CDCENTCUST'], $rateio['VRRATCCCTREC']
                );

                $this->entityManager->persist($ratctreceber);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Deleta Rateio Conta a Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @param $nrseqvenrec
     * @param $nrparcelarec
     * @param $cdcentcust
     * @throws \Exception
     */
    public function deletaRateioConta(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Rateio Conta Receber
            /** @var Ratctreceber $ratctreceber */
            $ratctreceber = $this->rateioContaFactory->buscaRateioConta(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
            );

            // Verifica Existência Rateio Conta
            if($ratctreceber) {
                // Guarda Filial/Valor Rateio Conta Receber
                $cdfiratctrec = $ratctreceber->getCdfiratctrec();
                $vrratccctrec = $ratctreceber->getVrratccctrec();

                // Deleta Rateio Conta Receber
                $this->entityManager->remove($ratctreceber);
                $this->entityManager->flush();

                // Busca Centro de Custo do Maior Valor Rateio Conta Receber
                $cdcentcust = $this->rateioContaFactory->buscaCentroCustoMaiorValorRateioConta(
                    $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdfiratctrec, $vrratccctrec
                );

                $cdcentcust = $cdcentcust[0]['CDCENTCUST'];

                // Verifica Exintência Maior Valor Rateio Conta Receber
                if ($cdcentcust) {
                    // Busca Maior Valor Rateio Conta Receber
                    /** @var Ratctreceber $ratctreceber */
                    $ratctreceber = $this->rateioContaFactory->buscaRateioConta(
                        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenrec, $nrparcelarec, $cdcentcust
                    );

                    // Atualiza Maior Valor Rateio Conta Receber
                    $ratctreceber = $this->rateioContaFactory->atualizaMaiorValorRateioConta(
                        $ratctreceber, $vrratccctrec
                    );

                    $this->entityManager->persist($ratctreceber);
                }
            }

            // Registra Rateio Excluído
            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

} 